#! python
from subprocess import call
import os
from ESTCommon import *

msg_CA = \
    """
#################################################################
####(Re)creating the directory structure and initial files" 
####for the CA: %s
#################################################################
"""
msg_int_cert = \
    """
#################################################################
####(Re)creating Level %d Intermediate certificate (Chain %d)
#################################################################
"""
msg_endpnt_crt = \
    """
#################################################################
####(Re)creating an initial peer certificate for an endpoint to 
#### use
#################################################################
"""


def createCA(CREATECA_CASUBJ,
             CREATECA_CADIR,
             CREATECA_CACERT,
             CREATECA_CAPRIVDIR,
             CREATECA_CASERIAL,
             CREATECA_CADATABASE,
             CREATECA_CANEWCERTSDIR,
             CREATECA_CAPRIVKEY,
             CREATECA_NEWKEY_PARAM,
             CREATECA_ECPARAMSFILE,
             ):
    print(msg_CA % (CREATECA_CASUBJ))
    os.makedirs(CREATECA_CADIR)
    os.makedirs(CREATECA_CAPRIVDIR)
    os.makedirs(CREATECA_CANEWCERTSDIR)
    with open(CREATECA_CASERIAL, "w") as serial_file:
        serial_file.write("01")
    touch(CREATECA_CADATABASE)
    system("%s ecparam -name prime256v1 -out %s" %
           (openssl_cmd, CREATECA_ECPARAMSFILE))
    iferrorlogandexit("Unable to build ECPARAMS file", 1)
    system("%s req -new -x509 -sha256 -extensions v3_ca %s -keyout %s -out %s -days 365 -nodes -subj \"%s\" -config %s" %
           (openssl_cmd, CREATECA_NEWKEY_PARAM, CREATECA_CAPRIVKEY, CREATECA_CACERT, CREATECA_CASUBJ, EST_OPENSSL_CACNF))
    iferrorlogandexit("Unable to create \"%s\" CA cert" % (CREATECA_CASUBJ), 1)
    system("%s x509 -sha256 -in %s" % (openssl_cmd, CREATECA_CACERT))


def createIntermediateCAs(TMP_CADIR,
                          TMP_CACERT,
                          TMP_CAPRIVKEY,
                          TMP_CAPRIVKEYPARAM,
                          TMP_NEWKEY_CMD,
                          TMP_FINAL_SUBJ,
                          TMP_FINAL_CACNF,
                          TMP_INT_CACNF,
                          TMP_INT_SUBJ,
                          TMP_FINAL_CERT,
                          TMP_FINAL_PRIVKEY,
                          CHAIN_FILE,
                          NUM_INTER,
                          CHAIN_NUM,
                          EXT_FILE="./ext.cnf"):
    TMP_CERTANDKEY = "%s/private/certandkey%s-%s" % (
        TMP_CADIR, CHAIN_NUM, TMP_FINAL_CERT)
    TMP_CERTREQ = "%s/est.req" % (TMP_CADIR)
    with open(CHAIN_FILE, "w") as fileout:
        with open(TMP_CACERT) as filein:
            fileout.write(filein.read())
    if(NUM_INTER >= 0):
        for i in range(NUM_INTER):
            TMP_PRIVKEY = "%s/private/cakey%d-%d.pem" % (
                TMP_CADIR, CHAIN_NUM, i)
            TMP_CERT = "%s/cacert%d-%d.crt" % (TMP_CADIR, CHAIN_NUM, i)
            TMP_SUBJ = "%s%d-%d" % (TMP_INT_SUBJ, CHAIN_NUM, i)
            if not os.path.isfile(TMP_PRIVKEY):
                print(msg_int_cert % (i, CHAIN_NUM))
                system("%s req -new -sha256 -nodes -out %s %s -keyout %s -subj %s -config %s" %
                       (openssl_cmd, TMP_CERTREQ, TMP_NEWKEY_CMD, TMP_PRIVKEY, TMP_SUBJ, TMP_INT_CACNF))
                iferrorlogandexit(
                    "Unable to create est SUDI Int Lvl %d CSR" % (i), 1)
                system("%s ca -md sha256 -out %s -batch -config %s -infiles %s" %
                       (openssl_cmd, TMP_CERT, TMP_INT_CACNF, TMP_CERTREQ))
                iferrorlogandexit(
                    "Unable to create est SUDI Int Lvl %d certificate" % (i), 1)
            system("%s x509 -sha256 -in %s -text" % (openssl_cmd, TMP_CERT))
            os.environ["COUNT"] = "%d-%d" % (CHAIN_NUM, i)
            TMP_CACERT = TMP_CERT
            TMP_CAPRIVKEY = TMP_PRIVKEY
            with open(CHAIN_FILE, "a") as outfile:
                with open(TMP_CACERT) as infile:
                    outfile.write(infile.read())
        print(msg_endpnt_crt)
        system("%s req -new -sha256 -nodes -out %s %s -keyout %s -subj %s -config %s" %
               (openssl_cmd, TMP_CERTREQ, TMP_NEWKEY_CMD, TMP_FINAL_PRIVKEY, TMP_FINAL_SUBJ, TMP_FINAL_CACNF))
        iferrorlogandexit("Unable to create est server CSR", 1)
        print(TMP_FINAL_CERT)
        system("%s ca -md sha256 -out %s -batch -config %s -extfile %s -infiles %s" %
               (openssl_cmd, TMP_FINAL_CERT, TMP_FINAL_CACNF, EXT_FILE, TMP_CERTREQ))
        iferrorlogandexit("Unable to create est server certificate", 1)
        del os.environ["COUNT"]
        system("%s x509 -sha256 -in %s -text" % (openssl_cmd, TMP_FINAL_CERT))
        combinefiles(TMP_FINAL_CERT, TMP_FINAL_PRIVKEY, TMP_CERTANDKEY)



#!python
import shutil
from certgen import *
msg_start_warn = """\
#################################################################
SECURITY CONSIDERATIONS - NOTE WELL
The sample scripts used to handle EST operations are NOT
intended to provide a secure implementation. They have not
been evaluated for security, they have not had a Threat Model
reviewed, they are not particularly good about cleaning up after
themselves and they assume the data exchanged is well formed
if the cryptographic checks pass.

In short: They are not to be trusted. They provide a functional
implementation only.

Continuing will completely erase/destroy/nuke the existing sudiCA\
"""
msg_destroy_alert = """\
Nuking from orbit!
#################################################################\
"""

msg_create_CA = """\
#################################################################
####(Re)creating an initial root SUDI CA certificate
#################################################################\
"""
msg_done_line = """\
#################################################################\
"""
msg_server_combine = """\
#################################################################
####Creating combined trusted cert file for server\
"""
msg_sudi_combine = """\
#################################################################
####Creating combined trusted cert file for client\
"""
msg_huge_combine = """\
#################################################################
####Creating combined trusted cert files for huge cert test\
"""


EST_OPENSSL_SUDICADIR = "sudiCA"
EST_OPENSSL_SUDICACNF = "estExampleSUDICA.cnf"
EST_OPENSSL_INT_SUDICACNF = "estExampleIntSUDICA.cnf"
EST_OPENSSL_SUDICACERT = "%s/cacert.crt" % EST_OPENSSL_SUDICADIR
EST_OPENSSL_SUDICAPRIVDIR = "%s/private" % EST_OPENSSL_SUDICADIR
EST_OPENSSL_SUDICANEWCERTSDIR = "%s/newcerts" % EST_OPENSSL_SUDICADIR
EST_OPENSSL_SUDICADATABASE = "%s/index.txt" % EST_OPENSSL_SUDICADIR
EST_OPENSSL_SUDICASERIAL = "%s/serial" % EST_OPENSSL_SUDICADIR
EST_OPENSSL_SUDICAPRIVKEY = "%s/cakey.pem" % EST_OPENSSL_SUDICAPRIVDIR
EST_OPENSSL_SUDICAPRIVKEYPARAM = "%s/cakeyparam.pem" % EST_OPENSSL_SUDICAPRIVDIR
EST_OPENSSL_SUDICASUBJ = "/CN=estExampleSUDICA"

EST_OPENSSLCMD_SUDICAECPARAMSFILE = "%s/prime256v1.pem" % EST_OPENSSL_SUDICADIR
EST_OPENSSLCMD_SUDICANEWKEY_PARAM = "-newkey ec:%s" % EST_OPENSSLCMD_SUDICAECPARAMSFILE
EST_OPENSSL_INT_HUGE_SUDICACNF = "estExampleHugeIntSUDICA.cnf"
EST_OPENSSL_HUGE_SUDICASUBJ = "\"/CN=estExampleHugeSUDICA.verylongwow.cisco123.404neverexisted.z\""
EST_OPENSSLCMD_HUGE_SUDICANEWKEY_PARAM = "-newkey rsa:4096"

EST_SUDI_SUBJ = "/CN=127.0.0.1"
EST_SUDI_CERTREQ = "%s/estsudi.req" % EST_OPENSSL_SUDICADIR
EST_SUDI_CERT = "estsudi.crt"
EST_SUDI_PRIVKEY = "estsudi.pem"
EST_HUGE_SUDI_CERT = "estHugeSUDI.crt"
EST_HUGE_SUDI_PRIVKEY = "estHugeSUDI.pem"

EST_HUGER_SUDI_CERT = "estHugerSUDI.crt"
EST_HUGER_SUDI_PRIVKEY = "estHugerSUDI.pem"
EST_SUDI_CCAUTHZDB = "ccAuthz.db"

detectWindowsFlag()
print(msg_start_warn)
try:
    input("Press Enter to continue...")
except:
    print("")
print(msg_destroy_alert)
shutil.rmtree(EST_OPENSSL_SUDICADIR, ignore_errors=True)
print(msg_create_CA)
createCA(EST_OPENSSL_SUDICASUBJ,
         EST_OPENSSL_SUDICADIR,
         EST_OPENSSL_SUDICACERT,
         EST_OPENSSL_SUDICAPRIVDIR,
         EST_OPENSSL_SUDICASERIAL,
         EST_OPENSSL_SUDICADATABASE,
         EST_OPENSSL_SUDICANEWCERTSDIR,
         EST_OPENSSL_SUDICAPRIVKEY,
         EST_OPENSSLCMD_SUDICANEWKEY_PARAM,
         EST_OPENSSLCMD_SUDICAECPARAMSFILE)

createIntermediateCAs(EST_OPENSSL_SUDICADIR,
                      EST_OPENSSL_SUDICACERT,
                      EST_OPENSSL_SUDICAPRIVKEY,
                      EST_OPENSSL_SUDICAPRIVKEYPARAM,
                      EST_OPENSSLCMD_SUDICANEWKEY_PARAM,
                      EST_SUDI_SUBJ,
                      EST_OPENSSL_SUDICACNF,
                      EST_OPENSSL_INT_SUDICACNF,
                      EST_OPENSSL_SUDICASUBJ,
                      EST_SUDI_CERT,
                      EST_SUDI_PRIVKEY,
                      "sudi_chain.crt",
                      2,
                      1)

createIntermediateCAs(EST_OPENSSL_SUDICADIR,
                      EST_OPENSSL_SUDICACERT,
                      EST_OPENSSL_SUDICAPRIVKEY,
                      EST_OPENSSL_SUDICAPRIVKEYPARAM,
                      EST_OPENSSLCMD_HUGE_SUDICANEWKEY_PARAM,
                      EST_SUDI_SUBJ,
                      EST_OPENSSL_SUDICACNF,
                      EST_OPENSSL_INT_HUGE_SUDICACNF,
                      EST_OPENSSL_HUGE_SUDICASUBJ,
                      EST_HUGE_SUDI_CERT,
                      EST_HUGE_SUDI_PRIVKEY,
                      "huge_sudi_chain.crt",
                      7,
                      2)

createIntermediateCAs(EST_OPENSSL_SUDICADIR,
                      EST_OPENSSL_SUDICACERT,
                      EST_OPENSSL_SUDICAPRIVKEY,
                      EST_OPENSSL_SUDICAPRIVKEYPARAM,
                      EST_OPENSSLCMD_HUGE_SUDICANEWKEY_PARAM,
                      EST_SUDI_SUBJ,
                      EST_OPENSSL_SUDICACNF,
                      EST_OPENSSL_INT_HUGE_SUDICACNF,
                      EST_OPENSSL_HUGE_SUDICASUBJ,
                      EST_HUGER_SUDI_CERT,
                      EST_HUGER_SUDI_PRIVKEY,
                      "huger_sudi_chain.crt",
                      8,
                      2)

print(msg_server_combine)
with open("../trustedcerts.crt", "a") as fileout:
    with open("sudiCA/cacert.crt") as filein:
        fileout.write(filein.read())

print(msg_done_line)

print(msg_sudi_combine)
combinefiles("../estCA/cacert.crt", "sudi_chain.crt",
             "trustedcertswithsudichain.crt")
print(msg_done_line)

print(msg_huge_combine)
combinefiles("../estCA/cacert.crt", "huge_sudi_chain.crt",
             "trustedcertswithhugesudichain.crt")
combinefiles("../estCA/cacert.crt", "huger_sudi_chain.crt",
             "trustedcertswithhugersudichain.crt")
print(msg_done_line)

print("Resetting the est server password file")
with open("../estCA/estpwdfile", "w") as file:
    file.write("estuser:estrealm:36807fa200741bb0e8fb04fcf08e2de6")
